@@ -3,5 +3,7 @@
 Scalpel is a sound editor. It works only on Linux at this point, with
 a GTK frontend and an ALSA backend.
 
+Changelog for this release at: http://scalpelsound.online.fr/?p=388
+
 """
-from app import __version__
+from constants import __version__
