@@ -45,16 +45,22 @@ class Graph(object):
         self.changed()
 
     def update(self):
-        "Called when sound has changed."
         self._adjust_view()
         self.changed()
 
-    def move_to(self, value):
+    def set_view(self, start, end):
+        self._view_start = start
+        self._view_end = end
+        self.update()
+
+    def move_to(self, frame):
         "Moves the view start and keep the view length"
         l = self._view_end - self._view_start
-        self._view_start = value
-        self._view_end = value + l
-        self.changed()
+        self.set_view(frame, frame + l)
+
+    def center_on(self, frame):
+        l = self._view_end - self._view_start
+        self.set_view(frame - l * 0.5, frame + l * 0.5)
 
     def numframes(self):
         return len(self._sound.frames)
@@ -67,11 +73,9 @@ class Graph(object):
     def density(self):
         "Number of frames per pixel."
         number_frames_view = (self._view_end - self._view_start)
-        if number_frames_view < self._width:
-            # sound is too small to fill the width
+        d = float(number_frames_view) / self._width
+        if d < 1:
             d = 1
-        else:
-            d = float(number_frames_view) / self._width
         return d
 
     def frmtopxl(self, f):
@@ -92,7 +96,7 @@ class Graph(object):
             value = maxi
         return value
 
-    def _zoom(self, point, factor):
+    def _zoom(self, factor):
         """Expand or shrink view according to factor.
 
         0 < factor < 1     zoom in
@@ -111,27 +115,37 @@ class Graph(object):
         """
         l = self._view_end - self._view_start
         l = l * factor
-        self._view_start = point - l * 0.5
-        self._view_end = point + l * 0.5
-        self._adjust_view()
-        self.changed()
-
+        self._view_end = self._view_start + l
         
     def zoom_in(self):
         "Make view twice smaller, centering on the middle of the view."
         mid = self._view_start + (self._view_end - self._view_start) * 0.5
-        self._zoom(mid, 0.5)
-        
+        self._zoom(0.5)
+        self.center_on(mid)
+
     def zoom_out(self):
         "Make view twice larger, centering on the middle of the view."
-        mid = self._view_start + (self._view_end - self._view_start) * 0.5 
-        self._zoom(mid, 2)
+        mid = self._view_start + (self._view_end - self._view_start) * 0.5
+        self._zoom(2)
+        self.center_on(mid)
 
-    def zoom_fit(self):
+    def zoom_out_full(self):
         "Fit everything in the view."
-        self._view_start = 0
-        self._view_end = len(self._sound.frames)
-        self.changed()
+        self.set_view(0, len(self._sound.frames))
+
+    def is_zoomed_out_full(self):
+        start, end = self.view()
+        return start == 0 and end == len(self._sound.frames)
+
+    def zoom_in_on(self, pixel):
+        point = self.pxltofrm(pixel)
+        self.zoom_in()
+        self.move_to(point - pixel * self.density())
+
+    def zoom_out_on(self, pixel):
+        point = self.pxltofrm(pixel)
+        self.zoom_out()
+        self.move_to(point - pixel * self.density())
 
     def _scroll(self, factor):
         """Shift the view.
@@ -144,10 +158,7 @@ class Graph(object):
         """
         l = self._view_end - self._view_start
         l = int(l * factor)
-        self._view_start += l
-        self._view_end += l
-        self._adjust_view()
-        self.changed()
+        self.set_view(self._view_start + l, self._view_end + l)
 
     def scroll_left(self):
         self._scroll(-0.20)
@@ -158,8 +169,7 @@ class Graph(object):
     def set_width(self, width):
         "Set the number of values the graph must have."
         self._width = int(width)
-        self._adjust_view()
-        self.changed()
+        self.update()
         
     def channels(self):
         "Return the graph values."
@@ -260,56 +270,67 @@ def test_zoom():
     g = Graph(sound)
 
     g.set_width(4)
-    g._zoom(point=1.5, factor=1)
+    g._zoom(1)
+    g.center_on(1.5)
     o = g.channels()
     assert o == [[(1, 1), (2, 2), (3, 3), (4, 4)]]
     
-    g._zoom(point=0, factor=1)
+    g._zoom(factor=1)
+    g.center_on(0)
     o = g.channels()
     assert o == [[(1, 1), (2, 2), (3, 3), (4, 4)]]
 
-    g._zoom(point=6, factor=1)
+    g._zoom(1)
+    g.center_on(6)
     o = g.channels()
     assert o == [[(1, 1), (2, 2), (3, 3), (4, 4)]]
     
     g.set_width(2)
-    g._zoom(point=1.5, factor=0.5)
+    g._zoom(0.5)
+    g.center_on(1.5)
     o = g.channels()
     assert o == [[(2, 2), (3, 3)]]
 
-    g._zoom(point=1.5, factor=0.5)
+    g._zoom(factor=0.5)
+    g.center_on(1.5)
     g.set_width(4)
     o = g.channels()
     assert o == [[(1, 1), (2, 2), (3, 3), (4, 4)]]
 
     g.set_width(2)
-    g._zoom(point=0, factor=0.5)
+    g._zoom(0.5)
+    g.center_on(0)
     o = g.channels()
     assert o == [[(1, 1), (2, 2)]]
 
     g.set_width(4)
-    g._zoom(point=0, factor=0.25)
+    g._zoom(0.25)
+    g.center_on(0)
     o = g.channels()
     assert o == [[(1, 1), (2, 2), (3, 3), (4, 4)]]
     
     g.set_width(4)
-    g._zoom(point=4, factor=4)
+    g._zoom(4)
+    g.center_on(4)
     o = g.channels()
     assert o == [[(1, 1), (2, 2), (3, 3), (4, 4)]]
 
     g.set_width(3)
     data = numpy.array([1, 2, 3, 4, 5])
     sound.frames = data
-    g._zoom(point=2, factor=0.5)
+    g._zoom(0.5)
+    g.center_on(2)
     o = g.channels()
-    print o
     assert o == [[(2, 2), (3, 3), (4, 4)]]
 
-    g._zoom(point=2, factor=0.5)
-    g._zoom(point=2, factor=0.5)
+    g._zoom(factor=0.5)
+    g.center_on(2)
+    g._zoom(factor=0.5)
+    g.center_on(2)
     start, end = g._view_start, g._view_end
     
-    g._zoom(point=2, factor=0.5 * 0.5)
+    g._zoom(factor=0.5 * 0.5)
+    g.center_on(2)
     assert (start, end) == (g._view_start, g._view_end)
 
 def test_zoom_in():
@@ -332,6 +353,27 @@ def test_zoom_in():
     o = g.channels()
     assert o == [[(1, 1), (2, 2), (3, 3), (4, 4)]]
 
+def test_zoom_in_on():
+    import numpy
+    from mock import Mock, Fake
+    sound = Mock({"numchan": 1})
+    sound.changed = Fake()
+    data = numpy.array([1, 2, 3, 4])
+    sound.frames = data
+    g = Graph(sound)
+    g.set_width(2)
+
+    g.zoom_in_on(0)
+    assert g.channels() == [[(1, 1), (2, 2)]]
+
+    g.zoom_out()
+    g.zoom_in_on(1)
+    assert g.channels() == [[(2, 2), (3, 3)]]
+
+    g.zoom_out()
+    g.zoom_in_on(2)
+    assert g.channels() == [[(3, 3), (4, 4)]]
+
 def test_scroll():
     import numpy
     from mock import Mock, Fake
@@ -358,3 +400,4 @@ if __name__ == "__main__":
     test_zoom()
     test_zoom_in()
     test_scroll()
+    test_zoom_in_on()
