@@ -10,10 +10,12 @@ import sys
 sys.argv[0] = constants.__appname__
 
 import app
+import control
 from gtkwaveform import GraphView, GraphScrollbar
 from gtkfiledialog import FileDialog
 import copy
 import os.path
+import urllib
 import gtk
 gtk.gdk.threads_init()
 
@@ -78,6 +80,13 @@ class EditorWindow(gtk.Window):
         self.vbox.pack_end(self.statusbar, expand=False, fill=False)
         self.add(self.vbox)
         
+        # Setup drag and drop
+        TARGET_TYPE_TEXT = 80
+        self.drag_dest_set(gtk.DEST_DEFAULT_ALL,
+                           [("text/uri-list", 0, TARGET_TYPE_TEXT)],
+                           gtk.gdk.ACTION_COPY)
+        self.connect("drag_data_received", self._open_dropped_files)
+
         self.connect("delete-event", self.close)
         self._filename_update()
         self.ctrl.filename_changed.connect(self._filename_update)
@@ -96,6 +105,7 @@ class EditorWindow(gtk.Window):
                 <menuitem action="Open"/>
                 <menuitem action="Save"/>
                 <menuitem action="Save as"/>
+                <menuitem action="Save Selection as"/>
                 <separator/>
                 <menuitem action="Close"/>
                 <menuitem action="Quit"/>
@@ -130,10 +140,8 @@ class EditorWindow(gtk.Window):
               <toolitem action="ZoomIn"/>
               <separator/>
               <toolitem action="Start"/>
-              <toolitem action="Rewind"/>
               <toolitem action="Pause"/>
               <toolitem action="Play"/>
-              <toolitem action="Forward"/>
               <toolitem action="End"/>
               <separator/>
               <toolitem action="Cut"/>
@@ -169,6 +177,8 @@ class EditorWindow(gtk.Window):
                    ('Open', gtk.STOCK_OPEN, None, None, '', self.open),
                    ('Save', gtk.STOCK_SAVE, None, None, '', self.save),
                    ('Save as', gtk.STOCK_SAVE_AS, None, None, '',self.save_as),
+                   ('Save Selection as', gtk.STOCK_SAVE_AS,'Save Selection As',
+                                 '<Ctrl><Alt>s', None, self.save_selection_as),
                    ('Close', gtk.STOCK_CLOSE, None, None, '', self.close),
                    ('Quit', gtk.STOCK_QUIT, None, None, '', self.quit),
                    ('Play', gtk.STOCK_MEDIA_PLAY, None, 'p', '', self.play),
@@ -177,10 +187,6 @@ class EditorWindow(gtk.Window):
                                                               self.goto_start),
                    ('End', gtk.STOCK_MEDIA_NEXT, None, 'End', '',
                                                                 self.goto_end),
-                   ('Rewind', gtk.STOCK_MEDIA_REWIND, None, None, '',
-                                                                  self.rewind),
-                   ('Forward', gtk.STOCK_MEDIA_FORWARD, None, None, '',
-                                                                 self.forward),
                    ('Cut', gtk.STOCK_CUT, None, None, '', self.cut),
                    ('Copy', gtk.STOCK_COPY, None, None, '', self.copy),
                    ('Paste', gtk.STOCK_PASTE, None, None, '', self.paste),
@@ -205,6 +211,42 @@ class EditorWindow(gtk.Window):
 
         return uimanager
 
+    def _close(self):
+        try:
+            self.ctrl.close()
+        except control.FileNotSaved:
+            proceed = self._show_dialog_close()
+            if not proceed:
+                return False
+            self.ctrl.close(force=True)
+
+        self.destroy()
+        self._windows.remove(self)
+
+        if not self._windows:
+            self.quit()
+
+        return True
+
+    def _show_dialog_close(self):
+        dialog = gtk.MessageDialog(parent=self, type=gtk.MESSAGE_WARNING)
+        name = self.ctrl.filename() or "sound"
+        name = os.path.basename(name)
+        dialog.set_markup("<b>Save %s before closing?</b>" % name)
+        dialog.add_button("Close _without saving", gtk.RESPONSE_NO)
+        dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+        dialog.add_button(gtk.STOCK_SAVE_AS, gtk.RESPONSE_YES)
+        response = dialog.run()
+        dialog.destroy()
+        proceed = False
+        if response == gtk.RESPONSE_NO:
+            proceed = True
+        elif response == gtk.RESPONSE_YES:
+            saved = self.save_as()
+            if saved:
+                proceed = True
+        return proceed
+
     def _filename_update(self):
         filename = self.ctrl.filename()
         self._update_title(filename)
@@ -216,10 +258,22 @@ class EditorWindow(gtk.Window):
             title = os.path.basename(filename) + ' - ' + title
         self.set_title(title)
 
+    def _open_dropped_files(self, widget, context, x, y,
+                            selection, targetType, time):
+        """Open files dragged and dropped on the window."""
+        filenames = selection.data.split()
+        for filename in filenames:
+            # Extract the actual filename
+            prefix = 'file://'
+            if filename.startswith(prefix):
+                filename = filename[len(prefix):]
+            filename = urllib.unquote(filename)
+            self.ctrl.open(filename)
+
     def display_error(self, title, text):
         display_error(title, text, parent=self)
 
-    # -- Callbacks
+    # -- GTK Callbacks
 
     def __getattr__(self, name):
         """Redirect callbacks to the controller.
@@ -228,7 +282,7 @@ class EditorWindow(gtk.Window):
         not be passed to the controller method.
         
         """
-        if name in ["new", "save", "play", "pause", "rewind", "forward",
+        if name in ["new", "save", "play", "pause",
                     "goto_start", "goto_end", "select_all",
                     "cut", "copy", "paste", "mix", "undo", "redo",
                     "zoom_in", "zoom_out", "zoom_fit",
@@ -260,20 +314,27 @@ class EditorWindow(gtk.Window):
 
     def save_as(self, *args):
         filename = self.filedialog.get_filename(action='save')
+        saved = False
         if filename != None:
             self.ctrl.save_as(filename)
+            saved = True
+        return saved
+
+    def save_selection_as(self, *args):
+        # FIXME: title
+        filename = self.filedialog.get_filename(action='save')
+        if filename != None:
+            self.ctrl.save_selection_as(filename)
 
     def close(self, *args):
-        self.ctrl.close()
-        self.destroy()
-        self._windows.remove(self)
-
-        if not self._windows:
-            self.quit()
+        self._close()
+        # Tell GTK not to hide the window, we take care of that:
+        return True
 
     def quit(self, *args):
         for win in copy.copy(self._windows):
-            win.close()
+            if not win._close():
+                return
         gtk.main_quit()
 
 
