@@ -92,6 +92,8 @@ class GraphView(LayeredGraphView):
         self.layers.append(CursorLayer(self, cursor))
         MouseSelection(self, selection)
         MouseScroll(self, graph)
+        MouseMiddleClick(self, graph)
+        PointerStyle(self, selection)
 
 
 # -- Layers that can be added to LayeredGraphview.
@@ -275,6 +277,10 @@ class CursorLayer(object):
 
 # -- Mouse event listeners that act on models.
 #
+
+def near(x, y):
+    return abs(x - y) < 10
+
 class MouseScroll(object):
     """Listens for mouse wheel events and scroll a graph.
 
@@ -287,10 +293,21 @@ class MouseScroll(object):
         widget.connect("scroll_event", self.scroll_event)
 
     def scroll_event(self, widget, event):
-        if event.direction in (gtk.gdk.SCROLL_UP, gtk.gdk.SCROLL_LEFT):
+        MOD1 = event.state & gtk.gdk.MOD1_MASK
+        LEFT = event.direction is gtk.gdk.SCROLL_LEFT
+        RIGHT = event.direction is gtk.gdk.SCROLL_RIGHT
+        UP = event.direction is gtk.gdk.SCROLL_UP
+        DOWN = event.direction is gtk.gdk.SCROLL_DOWN
+
+        if LEFT or (UP and MOD1):
             self._graph.scroll_left()
-        elif event.direction in (gtk.gdk.SCROLL_DOWN, gtk.gdk.SCROLL_RIGHT):
+        elif RIGHT or (DOWN and MOD1):
             self._graph.scroll_right()
+        elif UP:
+            self._graph.zoom_in_on(event.x)
+        elif DOWN:
+            self._graph.zoom_out_on(event.x)
+
 
 class MouseSelection(object):
     """Listens for mouse events and select graph area.
@@ -310,48 +327,114 @@ class MouseSelection(object):
         widget.connect("button_release_event", self.button_release)
         widget.connect("motion_notify_event", self.motion_notify)
 
-    def near(self, x, y):
-        return abs(x - y) < 10
-
     def button_press(self, widget, event):
         if event.button == 1:
             self.pressed = True
             x = event.x
             start, end = self._selection.pixels()
+            # a double click resumes selection.
+            if event.type == gtk.gdk._2BUTTON_PRESS:
+                self._selection.pin(x)
             # extend towards left
-            if self._selection.selected() and self.near(start, x):
+            elif self._selection.selected() and near(start, x):
                 self._selection.pin(end)
                 self._selection.extend(x)
             # extend towards right
-            elif self._selection.selected() and self.near(end, x):
+            elif self._selection.selected() and near(end, x):
                 self._selection.pin(start)
                 self._selection.extend(x)
             # start fresh selection
             else:
                 self._selection.pin(x)
 
+    def button_release(self, widget, event):
+        if event.button == 1:
+            self.pressed = False
+
     def motion_notify(self, widget, event):
         if self.pressed:
             x = event.window.get_pointer()[0]
             self._selection.extend(x)
 
-        self.pointer_style(widget, event)
 
-    def pointer_style(self, widget, event):
-        style = None
-        x = event.window.get_pointer()[0]
-        if self._selection.selected():
-            start, end = self._selection.pixels()
-            if self.near(start, x):
-                style = gtk.gdk.Cursor(gtk.gdk.LEFT_SIDE)
-            elif self.near(end, x):
-                style = gtk.gdk.Cursor(gtk.gdk.RIGHT_SIDE)
-        self.widget.window.set_cursor(style)
+class MouseMiddleClick(object):
+    """Shift the wave display when the middle button is pressed."""
+    def __init__(self, widget, graph):
+        self.widget = widget
+        self.graph = graph
+        self.pressed = False
+        widget.add_events(gtk.gdk.BUTTON_PRESS_MASK |
+                          gtk.gdk.BUTTON_RELEASE_MASK |
+                          gtk.gdk.POINTER_MOTION_MASK |
+                          gtk.gdk.POINTER_MOTION_HINT_MASK)
+        widget.connect("button_press_event", self.button_press)
+        widget.connect("button_release_event", self.button_release)
+        widget.connect("motion_notify_event", self.motion_notify)
+
+    def button_press(self, widget, event):
+        if event.button == 2:
+            self.pressed = True
+            self._xlast = event.x
 
     def button_release(self, widget, event):
-        if event.button == 1:
+        if event.button == 2:
             self.pressed = False
 
+    def motion_notify(self, widget, event):
+        if self.pressed:
+            x = event.window.get_pointer()[0]
+            delta = self._xlast - x
+            self._xlast = x
+            start, _ = self.graph.view()
+            self.graph.move_to(start + delta * self.graph.density())
+
+
+class PointerStyle(object):
+    """Change the pointer style.
+
+    Show a hand when the middle click is pressed, otherwise show
+    special cursors if the pointer is next to a selection bound.
+
+    """
+    LEFT_SIDE = gtk.gdk.Cursor(gtk.gdk.LEFT_SIDE)
+    RIGHT_SIDE = gtk.gdk.Cursor(gtk.gdk.RIGHT_SIDE)
+    HAND = gtk.gdk.Cursor(gtk.gdk.HAND1)
+
+    def __init__(self, widget, selection):
+        self.widget = widget
+        self._selection = selection
+        self.pressed = False
+        widget.add_events(gtk.gdk.BUTTON_PRESS_MASK |
+                          gtk.gdk.BUTTON_RELEASE_MASK |
+                          gtk.gdk.POINTER_MOTION_MASK |
+                          gtk.gdk.POINTER_MOTION_HINT_MASK)
+        widget.connect("button_press_event", self.button_press)
+        widget.connect("button_release_event", self.button_release)
+        widget.connect("motion_notify_event", self.motion_notify)
+
+    def button_press(self, widget, event):
+        if event.button == 2:
+            self.pressed = True
+            self.widget.window.set_cursor(self.HAND)
+
+    def button_release(self, widget, event):
+        if event.button == 2:
+            self.pressed = False
+            self.widget.window.set_cursor(None)
+
+    def motion_notify(self, widget, event):
+        if not self.pressed:
+            style = None
+            x = event.window.get_pointer()[0]
+            if self._selection.selected():
+                start, end = self._selection.pixels()
+                if near(start, x):
+                    style = self.LEFT_SIDE
+                elif near(end, x):
+                    style = self.RIGHT_SIDE
+            self.widget.window.set_cursor(style)
+
+
 # -- Horizontal scrollbar, subclassed to control a Graph object.
 #
 class GraphScrollbar(gtk.HScrollbar):
@@ -396,14 +479,21 @@ class GraphScrollbar(gtk.HScrollbar):
             length = self._graph.numframes()
             start, end = self._graph.view()
             if start != end:
-                self._adjustment.upper = length
-                self._adjustment.value = start
-                self._adjustment.page_size = (end - start)
+                page_size = (end - start)
+                self._adjustment.set_all(value = start,
+                                         lower = 0,
+                                         upper = length,
+                                         page_increment = page_size,
+                                         step_increment = page_size / 5.,
+                                         page_size = page_size)
             else:
                 # empty sound
-                self._adjustment.upper = 1
-                self._adjustment.value = 0
-                self._adjustment.page_size = 1
+                self._adjustment.set_all(value = 0,
+                                         lower = 0,
+                                         upper = 1,
+                                         page_increment = 0,
+                                         step_increment = 0,
+                                         page_size = 1)
             self.inhibit = False
 
 
